/* Copyright 2001 TransGaming Technologies Inc. */

#ifndef DDRAW_D3DDEVICE_GL_H_INCLUDED
#define DDRAW_D3DDEVICE_GL_H_INCLUDED

#include "wine/d3dhalgl.h"

/* #define WIREFRAME */

#ifdef WIREFRAME
extern BOOL gWireframe;
#endif   

#define GL_D3D_PRIV(surf) ((GL_Direct3DDeviceImpl*)((surf)->private))

#define GL_D3D_PRIV_VAR(name,surf) \
	GL_Direct3DDeviceImpl* name = GL_D3D_PRIV(surf)

struct _DD_GL_data;

struct GL_D3D_saved_cmd
{
    struct GL_D3D_saved_cmd *next;
    D3DHAL_DP2COMMAND cmd;
    LPVOID data1, data2;
};

struct GL_D3D_state_block
{
    struct GL_D3D_saved_cmd *cmds;
    unsigned int list;
};

struct GL_Direct3DDeviceImpl_Part
{
    struct _DD_GL_data*gl_data;
    LPD3DHAL_D3DEXTENDEDCAPS lpCaps;
    LPVOID color_array;
    DWORD color_array_size;
    LPVOID vertex_array;
    DWORD vertex_array_size;
    DWORD glFlags;
    DWORD dp2_dwFlags;
    DWORD dp2_dwVertexType;
    LPVOID dp2_lpVertices;
    DWORD dp2_dwVertexOffset;
    DWORD dp2_dwVertexLength;
    LPDDRAWI_DDRAWSURFACE_LCL dp2_VertexBuffer;
    D3DDRAWPRIMITIVESTRIDEDDATA sd;
    GLDRAWPRIMITIVESTRIDEDDATA stride;
    struct GL_D3D_saved_cmd *cmds;
    DWORD lights;
    struct GL_D3D_state_block *block;
    DWORD material_flags;
    DWORD current_color_material;
    DWORD texture_flags;
    struct GL_D3D_DriverCallbacks callbacks;
};

#define GLF_NEW_TRANSFORM	0x002
#define GLF_NEW_VIEW		0x004
#define GLF_NEW_TNL_MODE	0x008
#define GLF_NEW_LIGHTING	0x010 /* lighting & material renderstates */
#define GLF_NEW_TEXCOORD_TRANSFORM	0x020
#define GLF_USE_VERTEX_ARRAYS	0x040
#define GLF_STRIDED_DATA	0x080
#define GLF_VERTEX_BUFFER	0x100

/* values for material_flags */
#define GLMF_DO_DIFFUSE		0x001
#define GLMF_DO_SPECULAR	0x002
#define GLMF_DO_AMBIENT		0x004
#define GLMF_DO_EMISSIVE	0x008
#define GLMF_DIFFUSE_DIRTY	0x010
#define GLMF_SPECULAR_DIRTY	0x020
#define GLMF_AMBIENT_DIRTY	0x040
#define GLMF_EMISSIVE_DIRTY	0x080
#define GLMF_SHININESS_DIRTY	0x100
#define GLMF_COLOR_MATERIAL	0x200

#define GLMF_ALL_MATERIAL_DIRTY \
(GLMF_DIFFUSE_DIRTY|GLMF_SPECULAR_DIRTY|GLMF_AMBIENT_DIRTY|GLMF_EMISSIVE_DIRTY\
|GLMF_SHININESS_DIRTY)

/* values for texture_flags */
#define GLTF_DEFAULT_TEXCOORD_TRANSFORM	0x01


#define GL_FVF_NEW_TNL_BITS (D3DFVF_POSITION_MASK|D3DFVF_NORMAL)
#define GL_CHECK_NEW_TNL(x,y) (((x) ^ (y)) & GL_FVF_NEW_TNL_BITS)

#define GLD(priv) ((priv)->gl.gl_data)
#define HAVE_GL_EXT(priv,ext) (GLD(priv)->gl_ext[ext])

#define CHECK_GL_ERROR(x) do { GLenum err; err = glGetError(); if (err) ERR("glGetError returns %08x for %s\n", err, x); } while (0)

#define D3DTSS_TCI_MASK 0xFFFF0000U
#define D3DTTFF_COUNT_MASK 0xFFU

typedef struct
{
    struct GL_Direct3DDeviceImpl_Part gl;
} GL_Direct3DDeviceImpl;

LPVOID WINAPI GL_Direct3DDevice_Construct(IDirect3DDeviceImpl* This);
void WINAPI GL_Direct3DDevice_Destruct(LPVOID ctx);

/* gllight.c */
HRESULT D3D_GL_create_light(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT D3D_GL_set_light(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
void GL_LoadLightPositions(IDirect3DDeviceImpl* This, BOOL force);

/* glprim.c */
HRESULT D3D_GL_draw(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT D3D_GL_clear(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);

/* glstate.c */
HRESULT D3D_GL_render_state(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT D3D_GL_viewport_info(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT D3D_GL_z_range(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT D3D_GL_set_material(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT D3D_GL_set_transform(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);

/* gltex.c */
HRESULT D3D_GL_texture_stage_state(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT D3D_GL_texblt(IDirect3DDeviceImpl* This, LPD3DHAL_DP2COMMAND cmd, LPVOID data1, LPVOID data2);
HRESULT WINAPI GL_Direct3DDevice_validate_device(LPVOID ctx, LPDWORD lpdwPasses);

#endif
